import logging
from flexget.plugin import *
from flexget.utils import qualities

log = logging.getLogger('metainfo_quality')


class MetainfoQuality(object):
    """
    Utility:

    Set quality attribute for entries.
    """

    def validator(self):
        from flexget import validator
        return validator.factory('boolean')

    def on_feed_metainfo(self, feed):
        # check if disabled (value set to false)
        if 'metainfo_quality' in feed.config:
            if not feed.config['metainfo_quality']:
                return
        for entry in feed.entries:

            # helper function, iterate entry fields in certain order
            def field_order(x):
                order = ['title', 'description']
                return order.index(x[0]) if x[0] in order else len(order)
            for field_name, field_value in sorted(entry.items(), key=field_order):
                if not isinstance(field_value, basestring):
                    continue
                # ignore some fields ...
                if field_name in ['feed', 'uid']:
                    continue
                quality = qualities.parse_quality(field_value)
                if quality > qualities.UnknownQuality():
                    # if we find a quality in this field, stop searching
                    break
            entry['quality'] = quality.name
            log.log(5, 'Found quality %s (%s) for %s from field %s' % \
                (entry['quality'], quality, entry['title'], field_name))
            
register_plugin(MetainfoQuality, 'metainfo_quality', builtin=True)
